//go:build !disable_format_mp3 && !cgo

package mp3

import (
	"bytes"
	"git.gammaspectra.live/S.O.N.G/Kirika/audio"
	mp3Lib "github.com/hajimehoshi/go-mp3"
	"io"
	"unsafe"
)

const BlockSize = 1024 * 128

type Format struct {
}

func NewFormat() Format {
	return Format{}
}

func (f Format) Name() string {
	return "mp3"
}

func (f Format) DecoderDescription() string {
	return "hajimehoshi/go-mp3"
}

func (f Format) Open(r io.ReadSeekCloser) (audio.Source, error) {
	decoder, err := mp3Lib.NewDecoder(r)
	if err != nil {
		return nil, err
	}

	source := audio.NewInt16Source(16, decoder.SampleRate(), 2)

	go func() {
		defer source.Close()
		samples := make([]int16, BlockSize*2)
		const SizeofInt16 = int(unsafe.Sizeof(int16(0)))
		byteSlice := unsafe.Slice((*byte)(unsafe.Pointer(&samples[0])), len(samples)*SizeofInt16)
		for {
			n, err := decoder.Read(byteSlice)
			if err != nil {
				return
			}
			n /= SizeofInt16

			buf := make([]int16, n)
			copy(buf, samples[:n])
			source.IngestInt16(buf, 16)
		}
	}()

	return source, nil
}

func (f Format) Identify(peek []byte, extension string) bool {
	//match ID3 / sync header
	return peek[0] == 0xff && (peek[1]>>5) == 0b111 || (bytes.Compare(peek[:3], []byte{'I', 'D', '3'}) == 0 && extension != "flac") || extension == "mp3"
}
