package packetizer

import (
	"bufio"
	"io"
)

type FLACPacket struct {
	mode            KeepMode
	sampleNumber    int64
	endSampleNumber int64
	data            []byte
}

func (p *FLACPacket) KeepMode() KeepMode {
	return p.mode
}

func (p *FLACPacket) GetStartSampleNumber() int64 {
	return p.sampleNumber
}

func (p *FLACPacket) GetEndSampleNumber() int64 {
	return p.endSampleNumber
}

func (p *FLACPacket) GetData() []byte {
	return p.data
}

func (p *FLACPacket) Category() int64 {
	return 0
}

func NewFLACPacketizer(reader io.Reader) *FLACPacketizer {
	return &FLACPacketizer{reader: bufio.NewReader(reader)}
}

func (o *FLACPacketizer) Read(p []byte) (n int, err error) {
	n, err = o.reader.Read(p)
	if n > 0 {
		o.buffer = append(o.buffer, p[:n]...)
	}
	o.bytesRead += int64(n)
	return
}

func (o *FLACPacketizer) Seek(offset int64, whence int) (int64, error) {
	if offset == 0 && whence == io.SeekCurrent {
		return o.bytesRead, nil
	}
	return 0, io.ErrNoProgress
}

func (o *FLACPacketizer) Close() error {
	return nil
}

func (o *FLACPacketizer) getBuffer() (buf []byte) {
	buf = make([]byte, len(o.buffer))
	copy(buf, o.buffer)
	o.buffer = o.buffer[:0]
	return
}
