//go:build !disable_format_flac && !disable_codec_libflac && cgo

package packetizer

import (
	libflac "git.gammaspectra.live/S.O.N.G/goflac"
	"io"
)

type FLACPacketizer struct {
	reader    io.Reader
	stream    *libflac.Decoder
	offset    int64
	samples   int64
	bytesRead int64
	buffer    []byte
}

func (o *FLACPacketizer) GetPacket() Packet {
	var err error
	if o.stream == nil {
		o.stream, err = libflac.NewDecoderReader(o)
		if err != nil {
			return nil
		}
		return &FLACPacket{
			mode:            Keep,
			sampleNumber:    0,
			endSampleNumber: 0,
			data:            o.getBuffer(),
		}
	}

	frame, err := o.stream.ReadFrame()

	if err != nil {
		return nil
	}

	sampleNumber := o.samples
	nsamples := int64(len(frame.Buffer)) / int64(frame.Channels)
	o.samples += nsamples

	return &FLACPacket{
		mode:            Discard,
		sampleNumber:    sampleNumber,
		endSampleNumber: sampleNumber + nsamples,
		data:            o.getBuffer(),
	}
}
