//go:build disable_format_flac || disable_codec_libflac || !cgo

package packetizer

import (
	flac_parser "github.com/mewkiz/flac"
	"io"
)

type FLACPacketizer struct {
	reader    io.Reader
	stream    *flac_parser.Stream
	offset    int64
	samples   int64
	bytesRead int64
	buffer    []byte
}

func (o *FLACPacketizer) GetPacket() Packet {
	var err error
	if o.stream == nil {
		o.stream, err = flac_parser.NewSeek(o)
		if err != nil {
			return nil
		}
		return &FLACPacket{
			mode:            Keep,
			sampleNumber:    0,
			endSampleNumber: 0,
			data:            o.getBuffer(),
		}
	}

	frame, err := o.stream.ParseNext()

	if err != nil {
		return nil
	}

	sampleNumber := int64(frame.Num)
	if frame.HasFixedBlockSize {
		sampleNumber *= int64(frame.BlockSize)
	}
	return &FLACPacket{
		mode:            Discard,
		sampleNumber:    sampleNumber,
		endSampleNumber: sampleNumber + int64(frame.Subframes[0].NSamples),
		data:            o.getBuffer(),
	}
}
