//go:build !disable_format_opus && cgo

package packetizer

import (
	"git.gammaspectra.live/S.O.N.G/Kirika/audio/format/flac"
	"git.gammaspectra.live/S.O.N.G/Kirika/audio/format/opus"
	"git.gammaspectra.live/S.O.N.G/Kirika/test"
	"io"
	"os"
	"testing"
)

func TestPacketizeOgg(t *testing.T) {
	fp, err := os.Open(test.TestSingleSample24)
	if err != nil {
		t.Error(err)
		return
	}
	defer fp.Close()
	source, err := flac.NewFormat().Open(fp)
	if err != nil {
		t.Error(err)
		return
	}

	reader, writer := io.Pipe()

	go func() {
		defer writer.Close()
		err = opus.NewFormat().Encode(source, writer, nil)
		if err != nil {
			t.Error(err)
			return
		}
	}()

	pipePacketizer := NewOggPacketizer(reader)
	packetCount := 0
	packetBytes := 0
	for {
		packet := pipePacketizer.GetPacket()
		if packet == nil {
			break
		}
		packetCount++
		packetBytes += len(packet.GetData())
	}
	if packetCount != 395 {
		t.Errorf("Wrong Packet Count %d != %d", packetCount, 395)
	}
	if packetBytes != 5962688 {
		t.Errorf("Wrong Packet Bytes %d != %d", packetBytes, 5962688)
	}
}

func TestPacketizeOggOffset(t *testing.T) {
	fp, err := os.Open(test.TestSingleSample24)
	if err != nil {
		t.Error(err)
		return
	}
	defer fp.Close()
	source, err := flac.NewFormat().Open(fp)
	if err != nil {
		t.Error(err)
		return
	}

	reader, writer := io.Pipe()

	go func() {
		defer writer.Close()
		err = opus.NewFormat().Encode(source, writer, nil)
		if err != nil {
			t.Error(err)
			return
		}
	}()

	pipePacketizer := NewOggPacketizer(reader)
	packetCount := 0
	packetBytes := 0
	for {
		packet := pipePacketizer.GetPacket()
		if packet == nil {
			break
		}
		packetCount++
		if offsetablePacket, ok := packet.(OffsetablePacket); ok {
			packetBytes += len(offsetablePacket.GetDataOffset(1000))
		}
	}
	if packetCount != 395 {
		t.Errorf("Wrong Packet Count %d != %d", packetCount, 395)
	}
	if packetBytes != 5962688 {
		t.Errorf("Wrong Packet Bytes %d != %d", packetBytes, 5962688)
	}
}
